package com.broodsoft.grandeur.activity;

import java.io.IOException;
import java.net.URL;
import java.util.Collection;
import java.util.ResourceBundle;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javafx.collections.ObservableList;
import javafx.event.EventHandler;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.stage.Stage;
import javafx.stage.WindowEvent;

import com.broodsoft.grandeur.activity.naming.CssFileNamingStrategy;
import com.broodsoft.grandeur.activity.naming.FxmlFileNamingStrategy;
import com.broodsoft.grandeur.activity.naming.NoSuchResourceException;
import com.broodsoft.grandeur.activity.naming.ResourceBundleFileNamingStrategy;
import com.google.common.base.Optional;
import com.google.inject.Inject;

public class SimpleFxmlActivity extends AbstractActivity {
	private static final Logger LOGGER = LoggerFactory.getLogger(SimpleFxmlActivity.class);

	protected final FxmlFileNamingStrategy fxmlNaming;
	protected final ResourceBundleFileNamingStrategy resourceBundleNaming;
	protected final CssFileNamingStrategy cssNaming;

	protected SimpleFxmlActivity(Configuration config, String title) {
		super(config.baseConfig, title);

		this.fxmlNaming = config.fxmlNaming;
		this.resourceBundleNaming = config.resourceBundleNaming;
		this.cssNaming = config.cssNaming;
	}

	@Override
	public void create() {
		final FXMLLoader fxmlLoader = prepareFxmlLoader();
		final Parent viewRoot = loadViewFromFxml(fxmlLoader);
		initializeStage(viewRoot);
		addStylesheets();
	}

	@Override
	public void destroy() {}

	protected FXMLLoader prepareFxmlLoader() {
		final FXMLLoader fxmlLoader = new FXMLLoader();
		fxmlLoader.setLocation(getFxmlLocation());
		setResourceBundle(fxmlLoader);
		fxmlLoader.setController(this);
		return fxmlLoader;
	}

	protected Parent loadViewFromFxml(FXMLLoader fxmlLoader) {
		try {
			final Parent viewRoot = (Parent)fxmlLoader.load();
			return viewRoot;
		} catch(IOException ioe) {
			throw new IllegalStateException("Failed to load view from FXML", ioe);
		}
	}

	protected void initializeStage(Parent viewRoot) {
		final Stage stage = getStage();
		stage.getScene().setRoot(viewRoot);
		stage.addEventHandler(WindowEvent.WINDOW_CLOSE_REQUEST, createWindowCloseRequestHandler());
	}

	protected URL getFxmlLocation() {
		return fxmlNaming.getResource(this.getClass());
	}

	protected void setResourceBundle(FXMLLoader fxmlLoader) {
		Optional<ResourceBundle> resourceBundle = getResourceBundle();
		if(resourceBundle.isPresent()) {
			fxmlLoader.setResources(resourceBundle.get());
		}
	}

	protected Optional<ResourceBundle> getResourceBundle() {
		try {
			return Optional.of(resourceBundleNaming.getResource(this.getClass()));
		} catch(NoSuchResourceException e) {
			LOGGER.debug("No resource bundle found: "+e.getMessage());
			return Optional.absent();
		}
	}

	protected void addStylesheets() {
		final Optional<Collection<String>> activityStylesheets = getActivityStylesheets();
		if(activityStylesheets.isPresent()) {
			final ObservableList<String> stylesheets = getStage().getScene().getStylesheets();
			stylesheets.addAll(activityStylesheets.get());
		}
	}

	protected Optional<Collection<String>> getActivityStylesheets() {
		try {
			return Optional.<Collection<String>>of(cssNaming.getResource(this.getClass()));
		} catch(NoSuchResourceException e) {
			LOGGER.debug("No stylesheets found: "+e.getMessage());
			return Optional.absent();
		}
	}

	protected EventHandler<WindowEvent> createWindowCloseRequestHandler() {
		return new DefaultWindowCloseRequestHandler(this);
	}



	public static final class Configuration {
		final AbstractActivity.Configuration baseConfig;
		final FxmlFileNamingStrategy fxmlNaming;
		final ResourceBundleFileNamingStrategy resourceBundleNaming;
		final CssFileNamingStrategy cssNaming;

		@Inject
		public Configuration(
			AbstractActivity.Configuration baseConfig,
			FxmlFileNamingStrategy fxmlNaming,
			ResourceBundleFileNamingStrategy resourceBundleNaming,
			CssFileNamingStrategy cssNaming
		) {
			this.baseConfig = baseConfig;
			this.fxmlNaming = fxmlNaming;
			this.resourceBundleNaming = resourceBundleNaming;
			this.cssNaming = cssNaming;
		}
	}
}
