package com.broodsoft.grandeur.nexus;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.broodsoft.grandeur.activity.Activity;
import com.broodsoft.grandeur.intent.CreateActivityIntent;
import com.broodsoft.grandeur.intent.DestroyActivityIntent;
import com.broodsoft.grandeur.intent.NavigationIntent;
import com.broodsoft.grandeur.navigation.Navigator;
import com.broodsoft.guice.ApplicationInjectionModule;
import com.broodsoft.guice.BaseClassMatcher;
import com.broodsoft.id.Id;
import com.broodsoft.id.IdGenerator;
import com.broodsoft.id.numeric.NumericIdGenerator;
import com.google.common.eventbus.EventBus;
import com.google.common.eventbus.Subscribe;
import com.google.inject.AbstractModule;
import com.google.inject.Guice;
import com.google.inject.Injector;
import com.google.inject.Module;
import com.google.inject.Provides;
import com.google.inject.TypeLiteral;
import com.google.inject.matcher.Matchers;
import com.google.inject.spi.InjectionListener;
import com.google.inject.spi.TypeEncounter;
import com.google.inject.spi.TypeListener;

public class Nexus  {
	protected class InjectionModule extends AbstractModule {
		private final Logger LOGGER = LoggerFactory.getLogger(ApplicationInjectionModule.class);

		private final IdGenerator activityIdGenerator;

		protected InjectionModule() {
			activityIdGenerator = new NumericIdGenerator();
		}

		@Override
		protected void configure() {
			bind(Nexus.class).toInstance(Nexus.this);
			bind(EventBus.class).toInstance(nexusEventBus);
			bindListener(new BaseClassMatcher(Activity.class), new TypeListener() {
				@Override
				public <I> void hear(TypeLiteral<I> type, TypeEncounter<I> encounter) {
					encounter.register(new InjectionListener<I>() {
						@Override
						public void afterInjection(I injectee) {
							LOGGER.debug("Instantiated activity of type "+injectee.getClass().getCanonicalName());
							//Registration now happens explicitly in Nexus#instantiateActivity
//							registry.register((Activity)injectee);
						}
					});
				}
			});

			bindListener(Matchers.any(), new TypeListener() {
				@Override
				public <I> void hear(TypeLiteral<I> type, TypeEncounter<I> encounter) {
					encounter.register(new InjectionListener<I>() {
						@Override
						public void afterInjection(I injectee) {
							LOGGER.debug("Registering an instance of "+injectee.getClass().getCanonicalName()+" with the nexus event bus");
							nexusEventBus.register(injectee);
						}
					});
				}
			});
		}

		@Provides
		protected Id createActivityId() {
			return activityIdGenerator.generate();
		}
	}

	protected class Registry {
		private final Map<Id, Activity> activitiesById = new HashMap<>();

		public void register(Activity activity) {
			if(activitiesById.containsKey(activity.getId()))
				throw new IllegalArgumentException("There's already a registered Activity with ID " + activity.getId());

			activitiesById.put(activity.getId(), activity);
		}

		public Activity find(Id id) {
			return activitiesById.get(id);
		}

		public void unregister(Id id) {
			activitiesById.remove(id);
		}
	}

	protected final EventBus globalEventBus;
	protected final EventBus nexusEventBus;
	protected final Registry registry;
	protected final Injector injector;

	public Nexus(EventBus globalEventBus) {
		this(globalEventBus, Collections.<Module>emptyList());
	}

	public Nexus(EventBus globalEventBus, Collection<Module> modules) {
		this.globalEventBus = globalEventBus;
		this.nexusEventBus = new EventBus();
		this.nexusEventBus.register(this);
		this.registry = new Registry();

		Collection<Module> copyOfModules = new ArrayList<>(modules);
		copyOfModules.add(new InjectionModule());
		Module[] modulesAsArray = new Module[copyOfModules.size()];
		copyOfModules.toArray(modulesAsArray);
		this.injector = Guice.createInjector(modulesAsArray);
	}

	public Injector getInjector() {
		return injector;
	}

	public EventBus getNexusEventBus() {
		return nexusEventBus;
	}

	protected <A extends Activity> A instantiateActivity(Class<A> activityType) {
		A activity = injector.getInstance(activityType);
		registry.register(activity);
		return activity;
	}

	protected void destroyActivity(Id id) {
		destroyActivity(registry.find(id));
	}

	protected void destroyActivity(Activity activity) {
		activity.hide();
		activity.destroy();
		registry.unregister(activity.getId());
	}

	@Subscribe
	public void handleLaunchActivityIntent(CreateActivityIntent<Activity> intent) {
		Activity activity = instantiateActivity(intent.getActivityType());
		activity.create();
		activity.show();
		intent.getCreationHandler().handleCreated(activity);
	}

	@Subscribe
	public void handleDestroyActivityIntent(DestroyActivityIntent intent) {
		destroyActivity(intent.getActivityId());
		intent.getDestructionHandler().handleDestroyed();
	}

	@Subscribe
	public void handleNavigationIntent(NavigationIntent<Navigator> intent) {
		injector.getInstance(intent.getNavigatorType()).navigate();
	}
}
